# -*- coding: utf-8 -*-
""" Core scaffy code.

Core should not import from any other pacakges within scaffy but be used as a
base/common code for those.
"""
from __future__ import absolute_import, unicode_literals
